c
c***********************************************************************
      subroutine ehook(mw,iproc)
c***********************************************************************
c
c     This routine optionally accepts the mass-weighted cartesian 
c     coordinates and transforms them to cartesian coordinates (mw=1).
c     The energy is computed using the cartesian coordinates.  If the
c     mass-weighted option was chosen (mw=1), the coordinates are
c     transformed back to mass-weighted at the end.  
c
      implicit double precision (a-h,o-z)
c
      include 'param.inc'
      include 'percon.inc'
      include 'common.inc'
      include 'jaguarate.inc'
      common /ncgcom/ iop
      dimension fmat(n3tm,n3tm)
c     
c     If it is a Grate restart, read the energy     
c     
      irout=1
      if (irest.eq.1) call grest(irout,dummy)
      if (irout.eq.0) return
c
c     Optionally convert the mass-weighted coordinates to cartesian
c
      if (mw.eq.1) call trans(2,n3,amass,x,dx)
c
c     Compute the potential using the appropriate routines
c
c     If the energy corresponds to a stationary point, check if it has
c     already been calculated and stored in redinf.
c
        if (iop.eq.6) then                                              1128JC97
            i=5
            nacat=natom
        else 
          if (iop.gt.0) then
            call redinf(iop,fmat,ierr)
            if (ierr.eq.0) goto 1
            i=iop
          else
            i=-iop
            iop=-iop
          endif
            if (iop.eq.5) then
                nacat=natom
            else
                nacat=nratom(iop)
            endif
        endif
c
c     If the point has not been stored, it calculates it
c
        ich=icharge(i)
        imul=imultipl(i)
        call escener(natoms,ich,imul,natom,label,
     *iatsv,v,x,iop,nacat)
c
c     Optionally transfer back to mass-weighted coordinates
1     if (mw.eq.1) call trans(1,n3,amass,x,dx)
c
c     Put the energy at an appropriate zero of energy
      if (lezer0) v = v-ezer0     
c
c     new format, the bath term is a function of X instead of S
c
      IF (LBATH) THEN
        CALL EFFBATH(0)
      ENDIF
c     
c     If it is a Grate restart, write the energy
c     
      if (irest.eq.1) then
          irout=5
          call grest(irout,dummy)
      endif
c
      return
      end                                           
c
c*****
c
c***********************************************************************
      subroutine ghook(mw,iproc)
c***********************************************************************
c
c     This routine optionally accepts the mass-weighted cartesian 
c     coordinates and transforms them to cartesian coordinates (mw=1).
c     The cartesian coordinates are used when computing the energy and
c     first derivatives of the energy with respect to each cartesian 
c     coordinate.  If the mass-weighted option was chosen (mw=1), the 
c     coordinates and first derivatives are transformed back to mass-
c     weighted at the end.  
c
c
      implicit double precision (a-h,o-z)
c
      include 'param.inc'
      include 'percon.inc'
      include 'common.inc'
      include 'jaguarate.inc'
      common /ncgcom/ iop
      dimension fmat(n3tm,n3tm)
c     
c     If it is a Grate restart, read the energy and first der.     
c     
      irout=2
      if (irest.eq.1) call grest(irout,dummy)
      if (irout.eq.0) return
c
c     Optionally convert the mass-weighted coordinates to cartesian
c
      if (mw.eq.1) call trans(2,n3,amass,x,dx)
c
c     Compute the potential using the appropriate routines
c     If the calculation corresponds to a stationary point, check if it has
c     already been calculated and stored in redinf.
c
        if (iop.eq.6) then                                              1128JC97
            i=5
            nacat=natom
        else
            call redinf(iop,fmat,ierr)
            if (ierr.eq.0) goto 1
            i=iop
            if (iop.eq.5) then
                nacat=natom
            else
                nacat=nratom(iop)
            endif
        endif
c
c     If the point has not been stored, it calculates it
c
        ich=icharge(i)
        imul=imultipl(i)
        call escfirst(natoms,ich,imul,natom,label,
     *iatsv,v,x,dx,iop,nacat)
c
c     Put the energy at an appropriate zero of energy
c
1     if (lezer0) v = v-ezer0
c
c   calculate the derivative and energy, should be in Cartesian coord
c
      IF (LBATH) THEN
        CALL EFFBATH(1)
      ENDIF
c
c
c     Transfer back to mass-weighted coordinates
      if (mw.eq.1) call trans(1,n3,amass,x,dx)
c
c
c     If it is a Grate restart, write the energy and first der.
c
      if (irest.eq.1) then
          irout=6
          call grest(irout,dummy)
      endif
c
      return
      end                                           
c
c*****
c
c***********************************************************************
        SUBROUTINE ginpene(ICHARGEJ,IMULTIPLJ,NATOM,LABEL,iatsv,COORD,
     *  jtype,nacat,NATESC)
c***********************************************************************
c
c     This subroutine writes the input for an energy calculation
c
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
      include 'param.inc'
      include 'percon.inc'
      include 'jaguarate.inc'
      DIMENSION COORD(N3TM)                                             0701TA03
      DIMENSION LABEL(NATESC),IATSV(NATESC,8)                           1128JC97
C
      character *2 asymb(103)
c
c     array of atomic symbols
c
      data (asymb(i),i=1,103)  /
     *   'H ','He','Li','Be','B ','C ','N ','O ','F ','Ne',
     *   'Na','Mg','Al','Si','P ','S ','Cl','Ar',
     *   'K ','Ca','Sc','Ti','V ','Cr','Mn','Fe','Co','Ni','Cu',
     *        'Zn','Ga','Ge','As','Se','Br','Kr',
     *   'Rb','Sr','Y ','Zr','Nb','Mo','Tc','Ru','Rh','Pd','Ag',
     *        'Cd','In','Sn','Sb','Te','I ','Xe',
     *   'Cs','Ba','La','Ce','Pr','Nd','Pm','Sm','Eu','Gd','Tb',
     *        'Dy','Ho','Er','Tm','Yb','Lu','Hf','Ta','W ','Re',
     *        'Os','Ir','Pt','Au','Hg','Tl','Pb','Bi','Po','At','Rn',
     *   'Fr','Ra','Ac','Th','Pa','U','Np','Pu','Am','Cm','Bk',
     *        'Cf','Es','Fm','Md','No','Lr'/
C
      OPEN (UNIT=FU81,STATUS='NEW',FILE='esp.fu81')
C
c   Write the link0 commands
C
c      if (inlcer.gt.0) then
c        DO ILINIP=1,inlcer
c        WRITE(FU81,'(a80)') cerblk(ILINIP)
c        ENDDO
c      endif
C
C   Write the keywords line 
C
       DO ILINIP=1,inlene(jtype)
       WRITE(FU81,'(a80)') enerot(jtype,ILINIP)
       ENDDO
c
       WRITE(FU81,1001)
C
C   Write charge and multiplicity and atomic symbols and coordinates
C
C     WRITE(FU81,1011) ICHARGEJ,IMULTIPLJ
c
      i=1
1        if (jtype.ne.5.and.jtype.ne.6) then                            1128JC97
            inum = iatsv(i,jtype)
         else
            inum = i
         end if
         ilab = label(inum)
         write(FU81,1021)asymb(ilab),(coord(3*(inum-1)+j), j=1,3)
      i=i+1
      if (jtype.eq.5.or.jtype.eq.6) then                                1128JC97
               if (i.le.natom) then
                     goto 1
               endif
      else
               if (iatsv(i,jtype).ne.0) then
                     goto 1
               endif
      endif
      write(FU81,1026)
C     WRITE(FU81,1031)
C
c   Write the basis set
C
C      if (inlbas.gt.0) then
C        DO ILINIP=1,inlbas
C        WRITE(FU81,'(a80)') basblk(ILINIP)
C        ENDDO
C      endif
C     WRITE(FU81,1031)
C
      CLOSE (FU81)
C
      RETURN
C
1001  FORMAT (1x,'&zmat')
1021  FORMAT (1x,A2,5x,3(f14.8,3x))
1026  FORMAT (1x,'&')
C
         END
c
c*****
c
c***********************************************************************
        SUBROUTINE ginpfrs(ICHARGEJ,IMULTIPLJ,NATOM,LABEL,iatsv,COORD,
     *  jtype,nacat,NATESC)
c***********************************************************************
C
C   Subroutine that writes the appropriate Jaguar input file
c   for a first derivative calculation
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
      include 'param.inc'
      include 'percon.inc'
      include 'jaguarate.inc'
C     DIMENSION COORD(N3TM),DX(N3TM)                                    0701TA03
C     Variable DX was not used so it was taken out in Gaussrate 9.1.    0701TA03
      DIMENSION COORD(N3TM)                                             0701TA03
      DIMENSION LABEL(NATESC),IATSV(NATESC,8)                           1128JC97
C
      character * 2 asymb(103)
c
c     array of atomic symbols
c
      data (asymb(i),i=1,103)  /
     *   'H ','He','Li','Be','B ','C ','N ','O ','F ','Ne',
     *   'Na','Mg','Al','Si','P ','S ','Cl','Ar',
     *   'K ','Ca','Sc','Ti','V ','Cr','Mn','Fe','Co','Ni','Cu',
     *        'Zn','Ga','Ge','As','Se','Br','Kr',
     *   'Rb','Sr','Y ','Zr','Nb','Mo','Tc','Ru','Rh','Pd','Ag',
     *        'Cd','In','Sn','Sb','Te','I ','Xe',
     *   'Cs','Ba','La','Ce','Pr','Nd','Pm','Sm','Eu','Gd','Tb',
     *        'Dy','Ho','Er','Tm','Yb','Lu','Hf','Ta','W ','Re',
     *        'Os','Ir','Pt','Au','Hg','Tl','Pb','Bi','Po','At','Rn',
     *   'Fr','Ra','Ac','Th','Pa','U','Np','Pu','Am','Cm','Bk',
     *        'Cf','Es','Fm','Md','No','Lr'/
C
      OPEN (UNIT=FU81,STATUS='NEW',FILE='esp.fu81')
C
c   Write the link0 commands
C
C      if (inlcer.gt.0) then
C        DO ILINIP=1,inlcer
C        WRITE(FU81,'(a80)') cerblk(ILINIP)
C        ENDDO
C      endif
C
C   Write the keywords line 
C
       DO ILINIP=1,inlfrs(jtype)
       WRITE(FU81,'(a80)') firrot(jtype,ILINIP)
       ENDDO
       WRITE(FU81,1001)
C
C   Write charge and multiplicity and atomic symbols and coordinates
C
C     WRITE(FU81,1011) ICHARGEJ,IMULTIPLJ
C
      i=1
1        if (jtype.ne.5.and.jtype.ne.6) then                            1128JC97
            inum = iatsv(i,jtype)
         else
            inum = i
         end if
         ilab = label(inum)
         write(FU81,1021)asymb(ilab),(coord(3*(inum-1)+j), j=1,3)
      i=i+1
      if (jtype.eq.5.or.jtype.eq.6) then                                1128JC97
               if (i.le.natom) then
                     goto 1
               endif
      else
               if (iatsv(i,jtype).ne.0) then
                     goto 1
               endif
      endif
      WRITE(FU81,1026)
C
c   Write the basis set
C
C      if (inlbas.gt.0) then
C        DO ILINIP=1,inlbas
C        WRITE(FU81,'(a80)') basblk(ILINIP)
C        ENDDO
C      endif
C     WRITE(FU81,1031)
C     CLOSE (FU81)
C
      CLOSE (FU81)
      RETURN
C
1001  FORMAT (1x,'&zmat')
1021  FORMAT (1x,A2,5x,3(f14.8,3x))
1026  FORMAT (1x,'&')
C
         END
c
c*****
c
c***********************************************************************
        SUBROUTINE ginpsec(ICHARGEJ,IMULTIPLJ,NATOM,LABEL,IATSV,COORD,
     *  jtype,nacat,NATESC)
c***********************************************************************
C
C   Subroutine that writes the appropriate GAUSSIAN 03/98/94 input file
C   for performing a second derivative calculation
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      include 'param.inc'
      include 'percon.inc'
      include 'jaguarate.inc'
C
C     DIMENSION COORD(N3TM),DX(N3TM)                                    0701TA03
C     Variable DX was not used so it was taken out in Gaussrate 9.1.    0701TA03
      DIMENSION COORD(N3TM)                                             0701TA03
      DIMENSION LABEL(NATESC),IATSV(NATESC,8)                           1128JC97
C
      character * 2 asymb(103)
c
c     array of atomic symbols
c
      data (asymb(i),i=1,103)  /
     *   'H ','He','Li','Be','B ','C ','N ','O ','F ','Ne',
     *   'Na','Mg','Al','Si','P ','S ','Cl','Ar',
     *   'K ','Ca','Sc','Ti','V ','Cr','Mn','Fe','Co','Ni','Cu',
     *        'Zn','Ga','Ge','As','Se','Br','Kr',
     *   'Rb','Sr','Y ','Zr','Nb','Mo','Tc','Ru','Rh','Pd','Ag',
     *        'Cd','In','Sn','Sb','Te','I ','Xe',
     *   'Cs','Ba','La','Ce','Pr','Nd','Pm','Sm','Eu','Gd','Tb',
     *        'Dy','Ho','Er','Tm','Yb','Lu','Hf','Ta','W ','Re',
     *        'Os','Ir','Pt','Au','Hg','Tl','Pb','Bi','Po','At','Rn',
     *   'Fr','Ra','Ac','Th','Pa','U','Np','Pu','Am','Cm','Bk',
     *        'Cf','Es','Fm','Md','No','Lr'/
C
      OPEN (UNIT=FU81,STATUS='NEW',FILE='esp.fu81')
C
c   Write the link0 commands
C
C      if (inlcer.gt.0) then
C        DO ILINIP=1,inlcer
C        WRITE(FU81,'(a80)') cerblk(ILINIP)
C        ENDDO
C      endif
C
C   Write the keywords line 
C
       DO ILINIP=1,inlscr(jtype)
       WRITE(FU81,'(a80)') secrot(jtype,ILINIP)
       ENDDO
      WRITE(FU81,1001) 
C
C   Write charge and multiplicity and atomic symbols and coordinates
C
C     WRITE(FU81,1011) ICHARGEJ,IMULTIPLJ
c
c   Include only the atoms present on the desired system
c
      i=1
1        if (jtype.ne.5.and.jtype.ne.6) then                            1128JC97
c
c   If it is reactant or product, only some atoms are involved in the
c   calculation. 
c
            inum = iatsv(i,jtype)
         else
c
c   If it is the saddle point, all the atoms are involved
c
            inum = i
         end if
         ilab = label(inum)
         write(FU81,1021)asymb(ilab),(coord(3*(inum-1)+j), j=1,3)
      i=i+1
c
c   Check when it has to stop
c
      if (jtype.eq.5.or.jtype.eq.6) then                                1128JC97
               if (i.le.natom) then
                     goto 1
               endif
      else
               if (iatsv(i,jtype).ne.0) then
                     goto 1
               endif
      endif
      WRITE(FU81,1026)
C
c   Write the basis set
C
C      if (inlbas.gt.0) then
C        DO ILINIP=1,inlbas
C        WRITE(FU81,'(a80)') basblk(ILINIP)
C        ENDDO
C      endif
C     WRITE(FU81,1031)
      CLOSE (FU81)
C
      RETURN
C
1001  FORMAT (1x,'&zmat')
1021  FORMAT (1x,A2,5x,3(f14.8,3x))
1026  FORMAT (1x,'&')
C
      END
c
c*****
c
c***********************************************************************
      SUBROUTINE goutsec (IOP,V,COORD,TMPDX,FMAT,JTYPE,IATSV,
     *NATOM,NACAT,NATESC)
c***********************************************************************
C
C  Subroutine that extracts the geometry, energy, first and second 
C  derivatives information from the Jaguar output file
C  file (Test.FChk)
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      include 'param.inc'
      include 'percon.inc'
      include 'jaguarate.inc'
C
C     DIMENSION COORD(N3TM),DX(N3TM),FMAT(N3TM,N3TM)                    0701TA03
C     Variable DX was not used so it was taken out in Gaussrate 9.1.    0701TA03
      DIMENSION COORD(N3TM),FMAT(N3TM,N3TM)                             0701TA03
      DIMENSION TMPFMAT(N3TM,N3TM), IATSV(NATESC,8)                     1128JC97
      DIMENSION TMPDX(N3TM)
      CHARACTER*80 STRREAD
C     CHARACTER*80 CASE
      CHARACTER*25 SECID
      CHARACTER*16 FORCEID
      CHARACTER*6  STM
C     CHARACTER*16 FRCIDE1
C     CHARACTER*12 ENEID
C     CHARACTER*29 GEOMID
C     CHARACTER*1 CHARKEY(10)                                           0701TA03
C     Variable CHARKEY was not used so                                  0701TA03
C     it was taken out in Gaussrate 9.1.                                0701TA03
c
c     IOP = 0 obtain energy
c     IOP = 1 obtain energy and geometry
c     IOP = 2 obtain energy, geometry, and gradient
c     IOP = 3 obtain energy, geometry, gradient, and hessian
c
      DATA AUANG /1.889727D0/
C
      do i=1,n3tm
        tmpdx(i)=0
      enddo
C     do i=1,n3tm*n3tm
C       tmpfm(i)=0
C     enddo
C
C     It starts getting energy value
C
      CALL SYSTEM('./jextract energy')
      OPEN (UNIT=FU85,STATUS='UNKNOWN',FILE='Test.Fchk')
C
      READ (FU85,*,END=1002) V
      CLOSE (FU85)
      GOTO 2000

1002  WRITE (FU6,*) 'Error reading Jaguar output 1'               
      STOP
1003  WRITE (FU6,*) 'Error reading Jaguar output 2'
      STOP
1004  WRITE (FU6,*) 'Error reading Jaguar output 3'
      STOP
1014  WRITE (FU6,*) 'Error reading Jaguar output 3a'
      STOP
1024  WRITE (FU6,*) 'Error reading Jaguar output 3b'
      STOP
1005  WRITE (FU6,*) 'Error reading Jaguar output 4'
      STOP
c
c     Now, it will extract geometry information
c
2000  IF (IOP.LT.1) GOTO 9999
      CALL SYSTEM('./jextract coords')
      OPEN (UNIT=FU85,STATUS='UNKNOWN',FILE='Test.Fchk')
C
      Do 2100 ILIN=1,NACAT
        READ (FU85,*,END=1003) (COORD(3*(ILIN-1)+J), J=1,3)
        Do 2200 K=1,3
           COORD(3*(ILIN-1)+K)=COORD(3*(ILIN-1)+K)*AUANG
2200    CONTINUE
2100  CONTINUE
      CLOSE(FU85)
c
c     Now, it will look for the cartesian forces
c
c     IF (IOP.LT.2) GOTO 9999
c     CALL SYSTEM('./jextract forces')
c     OPEN (UNIT=FU85,STATUS='UNKNOWN',FILE='Test.Fchk')
c
c     Do 3100 ILIN=1,NACAT
c     READ (FU85,*,END=1004) (TMPDX(3*(ILIN-1)+J), J=1,3)
c       DO J=1,3
c        TMPDX(3*(ILIN-1)+J)=-1.0d0*TMPDX(3*(ILIN-1)+J)
c       ENDDO
c     WRITE (FU6,*)'Grad:',(TMPDX(3*(ILIN-1)+J), J=1,3)
c3100  CONTINUE
c      CLOSE(FU85)
     
      IF (IOP.LT.2) GOTO 9999
      INDFORCE=0
      FORCEID='forces'
      OPEN (UNIT=FU85,STATUS='UNKNOWN',FILE='esp.fu82')
 3200 READ (FU85,1041,END=1004) STRREAD
      IF (STRREAD(2:10).EQ.'Total cpu') GOTO 3300
      IF (STRREAD(3:8).NE.FORCEID) THEN
         GOTO 3200
      ELSE
         READ(FU85,*) 
         READ(FU85,*)
         READ(FU85,*)
         DO  ILIN=1,NACAT
            READ (FU85,*,END=1014) ID,STM,(TMPDX(3*(ILIN-1)+J), J=1,3)
              DO J=1,3
              TMPDX(3*(ILIN-1)+J)=-1.0d0*TMPDX(3*(ILIN-1)+J)
              ENDDO
         ENDDO
         INDFORCE=1
      ENDIF
      GOTO 3200
 3300 IF(INDFORCE.EQ.0) GOTO 1024
       CLOSE(FU85)
c
c     Now, it will look for the second derivative information
c     by means of the title '&hess' 
c
      IF (IOP.LT.3) GOTO 9999
      SECID='&hess'
      OPEN (UNIT=FU85,STATUS='UNKNOWN',FILE='Test.rst')
4000  READ (FU85,1041,END=1005) STRREAD
      IF (STRREAD(1:5).NE.SECID) THEN
         GOTO 4000
      ELSE
         NDEG=3*NACAT
         IF(MOD(NDEG,5).eq.0) then
            NTOT=NDEG/5
         ELSE
            NTOT=(NDEG/5)+1
         ENDIF
C        Write(fu6,*) 'NTOT= ',NTOT
         DO 4200 NBLOCK=1,NTOT
           READ(FU85,*) IBLOCK
           IEND=3*NACAT-IBLOCK+1
           K=0
           DO 4100 I=1,IEND
                READ(FU85,*) ID,(TMPFMAT(IBLOCK+I-1,IBLOCK+J),J=0,K)
                DO J = 0, K
                IF(IBLOCK+I-1.NE.IBLOCK+J) TMPFMAT(IBLOCK+J,IBLOCK+I-1)
     *          =TMPFMAT(IBLOCK+I-1,IBLOCK+J)
                ENDDO
                IF(K.LE.3) K=K+1
4100       CONTINUE
4200     CONTINUE
      ENDIF
C
C     DO I = 1, NDEG
C     DO J = 1, NDEG
C       IF(I.LT.J) TMPFMAT(I,J)=TMPFMAT(J,I)
C     ENDDO
C     ENDDO
C
      if (jtype.ne.5.and.jtype.ne.6) then                               1128JC97
C
C   Now, according to the ordering of the atoms at the saddle point,
C   we reorder the hessian information
C
         do l=1,nacat
         do m=1,nacat
           do i=1,3
           do j=1,3
             innx=3*(iatsv(l,jtype)-1)+i
             inny=3*(iatsv(m,jtype)-1)+j
             inox=3*(l-1)+i
             inoy=3*(m-1)+j
               FMAT(innx,inny)=TMPFMAT(inox,inoy)
           enddo
           enddo
         enddo
         enddo
      else
C
C   If we have calculated the saddle point Hessian, we don't need the
C   reordering
C
         do innx=1,n3tm
         do inny=1,n3tm
             FMAT(innx,inny)=TMPFMAT(innx,inny)
         enddo
         enddo
      endif
c
      CLOSE (FU85)
c
1041  FORMAT(A80)
9999  RETURN
      END
c
c*****
c
c***********************************************************************
       subroutine grest(irout,fmat)
c***********************************************************************
C
C    This subroutine compares the geometry with the one in the file and
C    if they are the same, it reads the properties of the point, so that
C    it will not be calculated again by means of Gaussian.
C    If the properties were stored, it returns irout=0
C
      implicit double precision (a-h,o-z)
      include 'param.inc'
      include 'percon.inc'
      include 'common.inc'
      include 'jaguarate.inc'
      common /ncgcom/ iop
      common /nccmcm/ iopc
      dimension fmat(n3tm,n3tm)
C     fallout=1.0D-8                                                    0825JC00
      icontrw=0
C
C     The firt thing we have to do is tp check if we only want to write
C     (then, irout>4) or we want reading and writing. In the last case:
C
1     if (irout.le.4) then
C
C     First, if the previous comparison was ok, we read everything and put 
C     it on an array for the next comparisons
C
        read (fu83,*,END=999) irread
        if (irread.eq.4) read(fu83,*,END=999)ip,icharge(ip),imultipl(ip)
        read (fu83,*,END=999) vread
        do icount=1,3*natom
        read (fu83,*,END=999) georead(icount)
        enddo
        do icount=1,3*natom
        read (fu83,*,END=999) dxread(icount)
        enddo
        if (irread.eq.3) then
        do icount=1,3*natom
        do jcount=1,3*natom
          read (fu83,*,END=999) fmread(icount,jcount)
        enddo
        enddo
      endif
C
C    If the stored point does the same calculation that we want to do,
C    continue. Otherwise, the program looks for a place where it is done
C
        if (irout.ne.irread) then
               if (icontrw.eq.0) then
                  icontrw=1
                  rewind fu83
                  goto 1
               else
                  goto 1
               endif
        endif
C
C    If it is a geometry optimization, it reads geometry, energy and 1st der.
C
        if (irout.eq.4) then
          v=vread
          do icount=1,natom*3
             x(icount)=georead(icount)
          enddo
          do icount=1,natom*3
             dx(icount)=dxread(icount)
          enddo
C
c   and now writes is to gwrite
c
          write (fu84,*) irread
          if (irread.eq.4) write(fu84,*)ip,icharge(ip),imultipl(ip)
          write (fu84,*) v
          do icount=1,3*natom
          write (fu84,*) x(icount)
          enddo
          do icount=1,3*natom
          write (fu84,*) dx(icount)
          enddo
c   
c   and sets irout=0 to indicate that everything was ok.
c       
          irout=0
          return
        else
c
C    Now, if it wasn't a geometry optimization, compare the geometres. 
C    If the one to be used in the calculations is different from the one in 
C    gwrite, it returns (it doesn't allow rotations or translations in order
C    to avoid problems along the reaction path). If IOP<5 (i.e., before
C    enetring the reaction path calculations) the comparison between
C    geometries is not performed. It is assumed that for reactants,
C    products and saddle point everything is exactly the same than
C    in the original calculation
C    The same for wells (IOP>6)
C
          if (iop.eq.6) then                                            1128JC97
          do icount=1,natom*3
            diff=x(icount)-georead(icount)
            if (abs(diff).gt.fallout) then  
               if (icontrw.eq.0) then
                  icontrw=1
                  rewind fu83
                  goto 1
               else
                  goto 1
               endif
            endif
          enddo
          endif
C          
C    If both geometries are the same, it reads the stored information
C    and writes to gwrite
C
          v=vread
          if (irout.gt.1) then
            do icount=1,natom*3
               dx(icount)=dxread(icount)
            enddo
          endif
          if (irout.eq.3) then
            do icount=1,natom*3
             do jcount=1,natom*3
               fmat(icount,jcount)=fmread(icount,jcount)
             enddo
            enddo
          endif
        write (fu84,*) irread
        write (fu84,*) vread
        do icount=1,3*natom
        write (fu84,*) georead(icount)
        enddo
        do icount=1,3*natom
        write (fu84,*) dxread(icount)
        enddo
        if (irout.eq.3) then
        do icount=1,3*natom
        do jcount=1,3*natom
         write (fu84,*) fmread(icount,jcount)
        enddo
        enddo
        endif
c   
c   and sets irout=0 to indicate that everything was ok.
c       
          irout=0
      endif
      return
c
c     if irout>4 (i.e., we only want writing):
c
      else
          irout=irout-4
          write (fu84,*) irout
          if (irout.eq.4) write(fu84,*)iop,icharge(iop),imultipl(iop)
          write (fu84,*) v
          do icount=1,3*natom
               write (fu84,*) x(icount)
          enddo
          do icount=1,3*natom
               write (fu84,*) dx(icount)
          enddo
          if (irout.eq.3) then
             do icount=1,3*natom
                do jcount=1,3*natom
                   write (fu84,*) fmat(icount,jcount)
                enddo
             enddo
           endif
      endif
999   if (icontrw.eq.0) then
         icontrw=1
         rewind fu83
         goto 1
      else
         RETURN
      endif
      RETURN
      END
c
c*****
c
c***********************************************************************
      subroutine hhook(xii,fmat,sstp,nend,mw,icent,iproc)
c***********************************************************************
c
c     This routine optionally accepts the mass-weighted cartesian 
c     coordinates and transforms them to cartesian coordinates (mw=1).
c     The cartesian coordinates are used when computing the energy and
c     second derivatives of the energy with respect to each cartesian 
c     coordinate.  If the mass-weighted option was chosen (mw=1), the 
c     coordinates and first derivatives are transformed back to mass-
c     weighted at the end.  
c
      implicit double precision (a-h,o-z)
c
      include 'param.inc'
      include 'percon.inc'
      include 'common.inc'
      include 'jaguarate.inc'
      common /ncgcom/ iop
      dimension xii(n3tm), fmat(n3tm,n3tm), tmpfmat(n3tm,n3tm)
      dimension bup(n3tm)
c     
c
c     Optionally convert the mass-weighted coordinates to cartesian
c
      if (mw.eq.1) call trans(2,n3,amass,x,dx)
c
c     If the hessian is calculated using polyrate:
c
      if (ispot.eq.0) then
          if (lgs(25).eq.0) then
             if (icent.eq.0) then
                 do 75 i=1,n3
 75                 bup(i) = x(i)
                 call seccen (bup,fmat,mw,sstp)
                 goto 1
             endif
             if (icent.eq.1) then
                 call seccep (mw,sstp,nend)
                 goto 1
             endif
          else if (lgs(25).eq.1) then
                 call deriv2(nend)
                 goto 1
          else if (lgs(25).eq.2) then
                 call derv24(nend)
                 goto 1
          endif
       endif
c
c     If it is calculated using Jaguar:
c     If it is a Jrate restart, read the energy, 1st and 2nd der.
c     
      irout=3
      if (irest.eq.1) call grest(irout,fmat)
      if (irout.eq.0) return
c
c     If the calculation corresponds to a stationary point, check if it has
c     already been calculated and stored in redinf.
c
        if (iop.eq.6) then                                              1128JC97
            i=5
            nacat=natom
        else 
            if (iop.eq.5) then
                nacat=natom
            else
                nacat=nratom(iop)
            endif
            i=iop
            call redinf(iop,fmat,ierr)
            if (ierr.eq.0) goto 149 
        endif
C              ich=icharge(i)
C              imul=imultipl(i)
               do 119 indfmex=1,n3tm
                do 129 indfmey=1,n3tm
                 fmat(indfmex,indfmey)=0.D0
129             continue
119            continue
               call escsec(natoms,ich,imul,natom,label,
     *iatsv,v,x,dx,fmat,iop,nacat)
c
c     Reorder the stored hessian (if we are dealing with reactants or
c     products) and convert into mass-weighted
c
149       if (iop.ne.5.and.iop.ne.6) then                               1128JC97
            do l=1,n3tm
            do m=1,n3tm
                tmpfmat(l,m)=fmat(l,m)
                fmat(l,m)=0.0D0
            enddo
            enddo
            do l=1,nratom(iop)
            do m=1,nratom(iop)
                  do k=1,3
                  do n=1,3
                     innx=3*(iatsv(l,iop)-1)+k
                     inny=3*(iatsv(m,iop)-1)+n
                     inox=3*(l-1)+k
                     inoy=3*(m-1)+n
                          fmat(inox,inoy)=tmpfmat(innx,inny)/
     *                     amass(ind(inox))/amass(ind(inoy))
                  enddo
                  enddo
              enddo
              enddo
           else
c
c   If it is the saddle point, we only have to convert the Hessian
c   into mass-weighted
c
c     for non-equilibrium solvation, need to add an extra harmonic term
c
            nend = natom*3
c
            if (LBATH.AND.LFOPT) THEN
              nend = nend + 1
              F1 = PI*PI*REDM/(16.0d0*FRICT)
              F2 = F1/FRICT
              DO I = 1,NEND-1
                 FMAT(NEND,I) = -F2*DIFFU(I)
                 FMAT(I,NEND) = FMAT(NEND,I)
              ENDDO
              FMAT(NEND,NEND)=F2
              DO I = 1,NEND-1
               DO J = 1,I
                 FMAT(I,J) = FMAT(I,J)+F2*DIFFU(I)*DIFFU(J)
                 FMAT(J,I) = FMAT(I,J)
               ENDDO
              ENDDO
            endif
c              do l=1,natom*3
c              do m=1,natom*3
              do l=1,nend
              do m=1,nend
                fmat(l,m)=fmat(l,m)/amass(ind(l))/amass(ind(m))
              enddo
              enddo
           endif
c
c     Put the appropriate zero of energy
c
      if (lezer0) v = v-ezer0   
c
c   calculate the derivative and energy, should be in Cartesian coord
c
      IF (LBATH) THEN
        CALL EFFBATH(2)
      ENDIF
c
 
c
c     Transfer back to mass-weighted coordinates
1     if (mw.eq.1) call trans(1,n3,amass,x,dx)
c
c     If it is a Grate restart, write the energy, 1st and 2nd der.
c     If ispot=0, the calculations of first derivatives or/and energies were 
c     already stored and we don't need to save the information about fmat.
c
      if (ispot.ne.0) then
        if (irest.eq.1) then
          irout=7
          call grest(irout,fmat)
        endif
      endif
c
      return
      end                                           
c
c*****
c
c***********************************************************************
         subroutine ohook (iop,iproc)
c***********************************************************************
c
c     This routine optionally accepts a initial guess to the cartesian
c     coordinates of the full system and returns the optimized geometry
c     and energy
C
c     If Gaussian is optimizing, all the optimization has been carried out 
c     in prep. Therefore, this subroutine only reads the and prints out
c     the results of the calculations in prep
c
c     The geometry is optimized for a given chemical species:
c         For IOP = 1, or 2; the chemical species is reactant 1 or 2.
c         For IOP = 3, or 4; the chemical species is product 1 or 2.
c         For IOP = 5, the chemical species is the saddle point.
c         For IOP = 7, or 8; the chemical species is reactants or 
c                            products wells.
c    comments corrected by JZ 02/17/2007
      implicit double precision (a-h,o-z)
c
      include 'param.inc'
      include 'percon.inc'
      include 'common.inc'
      include 'jaguarate.inc'
c     dimension fmat(n3tm,n3tm)                                         0701TA03
c     Variable fmat was not used so it was taken out in Gaussrate 9.1.  0701TA03
c     
c     if Polyrate is going to do the optimization, call newt.
c
      if (igpot.eq.0) then
                 call gsetp(iop)
                 if (nratom(iop).gt.1) then
                    call newt(iop)
                 else 
                    call ehook(0,iproc)
                 endif
                 if (lezer0) v = v-ezer0
                 return
      endif
      if (isup.eq.1) call gsetp(iop)
c
c     else:
c     If it is a Grate restart, read the geom, energ and 1st der.
c     
      irout=4
      if (irest.eq.1) call grest(irout,dummy)
      if (irout.eq.0) goto 3
c
      if (iop.le.2) then
           v=enresv
            if (isup.eq.1) then
                if (iop.eq.1) v=ereac1
                if (iop.eq.2) v=ereac2
            endif
           do m=1,n3tm
              x(m)=geresv(m)
             dx(m)=fsresv(m)
           enddo
      else if (iop.le.4) then
           v=enprsv
            if (isup.eq.1) then
                if (iop.eq.3) v=eprod1
                if (iop.eq.4) v=eprod2
            endif
           do m=1,n3tm
              x(m)=geprsv(m)
             dx(m)=fsprsv(m)
           enddo
      else if (iop.eq.7) then                                           1128JC97
           v=enwrsv                                                     1128JC97
           do m=1,n3tm                                                  1128JC97
              x(m)=gewrsv(m)                                            1128JC97
             dx(m)=fswrsv(m)                                            1128JC97
           enddo                                                        1128JC97
      else if (iop.eq.8) then                                           1128JC97
           v=enwpsv                                                     1128JC97
           do m=1,n3tm                                                  1128JC97
              x(m)=gewpsv(m)                                            1128JC97
             dx(m)=fswpsv(m)                                            1128JC97
           enddo                                                        1128JC97
      else
           v=enstsv
           do m=1,n3tm
              x(m)=gestsv(m)
             dx(m)=fsstsv(m)
           enddo
      endif
c    
c     Print out the results of the Gaussian optimization
c
c     Geometry
c
3     if (iop.eq.5) write(fu6,1006)
      write (fu6,1001)
      write (fu6,1011)
      inat=1
1     if (iop.ne.5) then
         inum = iatsv(inat,iop)
      else 
         inum = inat
      endif
         write(fu6,1021)inum,(x(3*(inum-1)+j), j=1,3)
      inat=inat+1
      if (iop.eq.5) then                                                1128JC97
           if (inat.le.natom) goto 1
      else
           if (iatsv(inat,iop).ne.0) goto 1
      endif
      write (fu6,1011)
c
c     Gradients
c
      write (fu6,1031)
      write (fu6,1011)
      inat=1
2     if (iop.ne.5) then                                                0615JC98
         inum = iatsv(inat,iop)
      else
         inum = inat
      endif
         write(fu6,1021)inum,(dx(3*(inum-1)+j), j=1,3)
      inat=inat+1
      if (iop.eq.5) then                                                1128JC97
           if (inat.le.natom) goto 2
      else
           if (iatsv(inat,iop).ne.0) goto 2
      endif
      write (fu6,1011)
c
c   calculate the derivative and energy, should be in Cartesian coord
c
      IF (LBATH) THEN
        CALL EFFBATH(1)
      ENDIF
c
c
c
c     If it is a Grate restart, write the geom., energy and 1st der.
c     
      if (irout.eq.0) return
      if (irest.eq.1) then
          irout=8
          call grest(irout,dummy)
      endif
      return
1001     format(/4x,'Geometry optimized using Jaguar (a.u.)')           0217JZ07
1006     format(//1X,32(1H*),' Saddle point ',32(1H*)//)
1011     format(/4x,55('-'),/)
1021     format(4x,i2,8x,3(f11.6,4x))
1031     format(/4x,'Gradients of the optimized structure (a.u.)')
      end                                           
c
c*****
c
c***********************************************************************
      subroutine prep
c***********************************************************************
c
c     This routine is the driver for all preparation routines needed 
c     to set up a jaguarate calculation.
c
c     This routine is called after all POLYRATE input has been read in
c     and before the table of input is written to fu6.  It is also
c     before any calculations are done.  This routine is used
c     to call all electronic structure preparation routines. 
c
      implicit double precision (a-h,o-z)
c
      include 'param.inc'
      include 'percon.inc'
      include 'common.inc'
      include 'jaguarate.inc'
      common /ncgcom/ iop
      common /nccmcm/ iopc
      common /eshscm/ fmat(n3tm,n3tm)
      dimension xcm(3)
      dimension nrs(8)
c
c     Check if it is the first call to this subroutine
c
      if (numci.eq.0) then
c
c     if it is, set up some variables and start the Gaussian 
c     optimization procedure (if selected) and the hessian 
c     calculations for reactant and products.
c
      ereac=0.D0
      ereac1=0.D0
      ereac2=0.D0
      eprod=0.D0
      eprod1=0.D0
      eprod2=0.D0
      do i=1,8
        nrs(i)=nratom(i)
      enddo
      nrs(5)=natom
      do i=1,8
        vjdep(i)=0.D0
        icharge(i)=0
        imultipl(i)=1
        ioptgr(i)=0
      enddo
      call setup
c
c     Set isavect. If it is set to zero, the computer time is reduced as
c     much as possible, but it can be dangerous for optimizing geometries,
c     calculating anharmonicities, and second (or higher) derivatives by means 
c     of Polyrate methods. If it is set to one, some calculations may be 
c     repeated, but it will be much safer for the above mentioned
c     calculations. If isavect=2, the calculation is completely safe,
c     no check for saving time is done and derivatives will be calculated
c     correctly.
c     isavect=1 has been removed, since it saves little computer time
c     and there is a certain risk of failure
c
      isavect=0
      do 3 inesp=1,8                                                    1128JC97
       if (inesp.eq.6) goto 3                                           1128JC97
       if (ispot.eq.0) isavect=3
       if (igpot.eq.0) then
        if (istatu(inesp).eq.0) then                                    0729YC97
               if (nrs(inesp).gt.1) then
                     isavect=2
                     goto 1
               endif
        endif
       endif
        if (iharm(inesp).ne.1) then
            do inm=1,nxmod(inesp)
             if (vharmr(inm,inesp).ne.'harmonic '.and.
     *           vharmr(inm,inesp).ne.'hinrot   ') then
                       isavect=3
             endif
            enddo
        endif
3     continue
c
c
1     numci=1 
      if (isavect.eq.2) then
             if (isup.eq.1) return
             if (isup.eq.0) then
                    write (FU6,1001)
                    stop 'prep 1'
             endif
      endif
      if (isavect.eq.3) then
             if (isup.eq.1) isavect=2
             if (isup.eq.0) then
                    write (FU6,1001)
                    stop 'prep 2'
             endif
      endif
C
C     Calculations for reactants
C
      if (istatu(1).ne.6.or.istatu(2).ne.6) then                        0725YC97
      do iopc = 1,2                                          
        if (irepr(iopc).ne.0) then
            if (istatu(iopc).ne.6) then                                 0729YC97
            if (nratom(iopc).gt.1) then
              if (ioptgr(iopc).eq.1.and.istatu(iopc).lt.2) then         0725YC97
                 call geoin(iopc,ircm)
C                if (ircm.eq.1) call rchmul(iopc)
              else
                 imultipl(iopc)=nedeg(3*iopc-2)                         0725YC97
                 do i=1,nratom(iopc)                                    0729YC97
                     inum = iatsv(i,iopc)                               0729YC97
                     do j = 2,0,-1                                      0729YC97
                        x(inum*3-j) = xr(inum*3-j,1)                    0729YC97
                     enddo                                              0729YC97
                 enddo                                                  0729YC97
              endif
                 if (isavect.ne.2) then
                   call secin(iopc)
                 else
                   itmp=iop
                   iop=iopc
                   call ehook(0,iproc)
                   iop=itmp
                 endif
            else
              if (ioptgr(iopc).eq.1) then
                 call enatin(iopc,ircm)
C                if (ircm.eq.1) call rchmul(iopc)
              else
                 itmp=iop
                 iop=-iopc
                 call ehook(0,iproc)
                 iop=itmp
              endif
            endif
            else                                                        0729YC97
              V = steng(iopc)                                           0729YC97
              do i=1,nratom(iopc)                                       0729YC97
              inum = iatsv(i,iopc)                                      0729YC97
                 do j = 2,0,-1                                          0729YC97
                    x(inum*3-j) = xr(inum*3-j,1)                        0729YC97
                 enddo                                                  0729YC97
              enddo                                                     0729YC97
            endif                                                       0729YC97
c         vjdep(iopc)=v
        ereac=ereac+V
        if (isup.eq.1) then
            if (iopc.eq.1) then                                         1201JC97
               ereac1=V
            else
               ereac2=V
            endif
        endif
        endif
      enddo
C
C     saves the result for the following calculations on these points
C
        if (isup.eq.0) then
            itmp=iop
            iop=5
            call ehook(0,iproc)
            iop=itmp
            ereac=v
        endif
       enresv=ereac
       if (isup.eq.0) ezer0=ereac
       do i=1,n3tm
       geresv(i)=x(i)
       xr(i,1)=x(i)
       x(i)=0.D0
       fsresv(i)=DX(i)
       DX(i)=0.D0
        do j=1,n3tm
        hsresv(i,j)=fmat(i,j)
        fmat(i,j)=0.d0
        enddo
       enddo
      endif                                                             0725YC97
C
C     Calculations for products
C
      if (istatu(3).ne.6.or.istatu(4).ne.6) then                        0725YC97
      do iopc = 3,4
        if (irepr(iopc).ne.0) then
            if (istatu(iopc).ne.6) then                                 0729YC97
            if (nratom(iopc).gt.1) then
              if (ioptgr(iopc).eq.1.and.istatu(iopc).lt.2) then         0725YC97
                 call geoin(iopc,ircm)
C                if (ircm.eq.1) call rchmul(iopc)
              else
                 imultipl(iopc)=nedeg(3*iopc-2)                         0725YC97
                 do i=1,nratom(iopc)                                    0729YC97
                     inum = iatsv(i,iopc)                               0729YC97
                     do j = 2,0,-1                                      0729YC97
                        x(inum*3-j) = xr(inum*3-j,3)                    0729YC97
                     enddo                                              0729YC97
                 enddo                                                  0729YC97
              endif
                 if (isavect.ne.2) then
                 call secin(iopc)
                 endif
            else
              if (ioptgr(iopc).eq.1) then
                 call enatin(iopc,ircm)
C                if (ircm.eq.1) call rchmul(iopc)
              else
                 itmp=iop
                 iop=-iopc
                 call ehook(0,iproc)    
                 iop=itmp
              endif
            endif
            else                                                        0729YC97
              V = steng(iopc)                                           0729YC97
              do i=1,nratom(iopc)                                       0729YC97
              inum = iatsv(i,iopc)                                      0729YC97
                 do j = 2,0,-1                                          0729YC97
                    x(inum*3-j) = xr(inum*3-j,3)                        0729YC97
                 enddo                                                  0729YC97
              enddo                                                     0729YC97
            endif                                                       0729YC97
c         vjdep(iopc)=v
      eprod=eprod+v
        if (isup.eq.1) then
            if (iopc.eq.3) then                                         1201JC97
               eprod1=V
            else
               eprod2=V
            endif
        endif
        endif
      enddo
        if (isup.eq.0) then
            itmp=iop
            iop=5
            call ehook(0,iproc)
            iop=itmp
            eprod=v
        endif
C
C     saves the result for the following calculations on these points
C
       enprsv=eprod
       do i=1,n3tm
       geprsv(i)=x(i)
       xr(i,3)=x(i)
       x(i)=0.D0
       fsprsv(i)=DX(i)
       dx(i)=0.D0
        do j=1,n3tm
        hsprsv(i,j)=fmat(i,j)
        fmat(i,j)=0.D0
        enddo
       enddo
      endif                                                             0725YC97
C
C     Calculations for wells                                            1128JC97
C
      if (istatu(7).ne.6.or.istatu(8).ne.6) then                        1128JC97
       do iopc = 7,8                                                    1128JC97
        if (irepr(iopc).ne.0) then                                      1128JC97
            if (istatu(iopc).ne.6) then                                 1128JC97
              if (ioptgr(iopc).eq.1.and.istatu(iopc).lt.2) then         1128JC97
                 call geoin(iopc,ircm)                                  1128JC97
C                if (ircm.eq.1) call rchmul(iopc)                       1128JC97
              else                                                      1128JC97
                 imultipl(iopc)=nedeg(3*iopc-2)                         1128JC97
                 do i=1,nratom(iopc)                                    1128JC97
                     inum = iatsv(i,iopc)                               1128JC97
                     do j = 2,0,-1                                      1128JC97
                        x(inum*3-j) = xr(inum*3-j,iopc)                 1128JC97
                     enddo                                              1128JC97
                 enddo                                                  1128JC97
              endif                                                     1128JC97
                 if (isavect.ne.2) then                                 1128JC97
                    call secin(iopc)                                    1128JC97
                 endif                                                  1128JC97
            else                                                        1128JC97
              V = steng(iopc)                                           1128JC97
              do i=1,nratom(iopc)                                       1128JC97
              inum = iatsv(i,iopc)                                      1128JC97
                 do j = 2,0,-1                                          1128JC97
                    x(inum*3-j) = xr(inum*3-j,iopc)                     1128JC97
                 enddo                                                  1128JC97
              enddo                                                     1128JC97
            endif                                                       1128JC97
        endif                                                           1128JC97
C
C     saves the result for the following calculations on these points
C
        if (iopc.eq.7) then                                             1128JC97
           enwrsv=V                                                     1128JC97
           do i=1,n3tm                                                  1128JC97
              gewrsv(i)=x(i)                                            1128JC97
              xr(i,7)=x(i)                                              1128JC97
              x(i)=0.D0                                                 1128JC97
              fswrsv(i)=DX(i)                                           1128JC97
              dx(i)=0.D0                                                1128JC97
              do j=1,n3tm                                               1128JC97
                hswrsv(i,j)=fmat(i,j)                                   1128JC97
                fmat(i,j)=0.D0                                          1128JC97
              enddo                                                     1128JC97
           enddo                                                        1128JC97
        else                                                            1128JC97
           enwpsv=V                                                     1128JC97
           do i=1,n3tm                                                  1128JC97
              gewpsv(i)=x(i)                                            1128JC97
              xr(i,7)=x(i)                                              1128JC97
              x(i)=0.D0                                                 1128JC97
              fswpsv(i)=DX(i)                                           1128JC97
              dx(i)=0.D0                                                1128JC97
              do j=1,n3tm                                               1128JC97
                hswpsv(i,j)=fmat(i,j)                                   1128JC97
                fmat(i,j)=0.D0                                          1128JC97
              enddo                                                     1128JC97
           enddo                                                        1128JC97
        endif                                                           1128JC97
       enddo                                                            1128JC97
      endif                                                             1128JC97
C
C     Calculations for starting geometry
C
      iopc=5
      if (irepr(5).eq.1) then                                           0811YC97
         if (ioptgr(iopc).eq.1.and.istatu(5).lt.2) then                 0725YC97
            call geoin(iopc,ircm)
C           if (ircm.eq.1) call rchmul(iopc)
         else
            imultipl(5)=nedeg(13)                                       0725YC97
                 do inatnop=1,n3tm
                 x(inatnop)=xr(inatnop,5)
                 enddo
         endif
c   
c   For avoiding problems related to reorientation, we will translate 
c   origin of coordinates to the center of masses of the optimized 
c   saddle point (energy, gradients and hessians will not change, but
c   it will be safer for the calculation of the reaction path).
c
           totm=0.D0
           do i=1,3
              xcm(i)=0.D0
           enddo
           do k=1,natom
             ams=amass(k*3)**2.0D0
             totm=totm+ams
             do l=1,3
               xcm(l)=xcm(l)+x((k-1)*3+l)*ams
             enddo
           enddo
           do k=1,natom
               do l=1,3
                  x((k-1)*3+l)=x((k-1)*3+l)-xcm(l)/totm
               enddo
           enddo
c
c   Now, it calculates the gradients and hessian for the saddle point
c
           if (istatu(5).lt.4) then                                     0725YC97
                 if (isavect.ne.2) then
                 call secin(iopc)
                 endif
           endif                                                        0725YC97
                 estar=v
C
C     saves the result for the following calculations on these points
C
       enstsv=estar
       do i=1,n3tm
       gestsv(i)=x(i)
       xr(i,5)=x(i)
       x(i)=0.D0
       fsstsv(i)=DX(i)
       dx(i)=0.D0
        do j=1,n3tm
        hsstsv(i,j)=fmat(i,j)
        fmat(i,j)=0.D0
        enddo
       enddo
      endif
      endif                                                             0811YC97
c
      return
1001  format (3x,'Gaussrate error: SUPERMOL keyword is only supported ',
     *'along with the',/,3x,'following options:',//,7x,'OHOOK option ',
     *'in OPTTS and OPTMIN',/,7x,'HHOOK option in HESSCAL',/,7x,'HAR',
     *'MONIC or HINROT approximation for vibrational modes')
      end                                           
c
c*****
c
c***********************************************************************
      subroutine prepj(jtype)
c***********************************************************************
c
c     This routine is the species dependent
c     driver for all preparation routines needed to set up an electronic 
c     structure calculation.  Any initializations that are dependent
c     on wheter the reactants, products, saddle point, or generalized
c     transition state is computed should be done here.  JTYPE specifies
c     the species:
c
c           JTYPE               Species
c             1                 Reactant #1
c             2                 Reactant #2
c             3                 Product #1
c             4                 Product #2
c             5                 Saddle point
c             6                 Generalized transition states
c             7                 Reactant well
c             8                 Product well
c
c     This routine is called after each respective call to stvarj.
c
c
      implicit double precision (a-h,o-z)
c
      include 'param.inc'
      include 'percon.inc'
      include 'common.inc'
      include 'jaguarate.inc'
      common /ncgcom/ iop
      common /eshscm/ fmat(n3tm,n3tm)
      if (jtype.ne.6) then
         call stlgsj(jtype)
         call stvarj(jtype)
      end if
      iop=jtype
      return
      end
C
C*****
C
C***********************************************************************
C      SUBROUTINE rchmul(jtype)
C***********************************************************************
C
C     This subroutine reads the charge and multiplicity from a
C     Gaussian formated Checkpoint file
C
C     IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C     include 'param.inc'
C     include 'percon.inc'
C     include 'common.inc'
C     include 'gaussrate.inc'
C     CHARACTER*80 STRREAD
C     CHARACTER*80 CASE
C     CHARACTER*6  CHARID
C     CHARACTER*12 MULTID
C
C     It starts looking for the charge by means of the title
C     'Charge'
C
C     OPEN (UNIT=FU85,STATUS='UNKNOWN',FILE='Test.FChk')
C
C     CHARID='charge'
C1015  READ (FU85,1041,END=1005) STRREAD
C     DO 120 ILINPOS=1,3
C         STRREAD=CASE(STRREAD)
C         IF (STRREAD(ILINPOS:ILINPOS+5).ne.charid) THEN
C                   GOTO 120
C            ELSE
C                   is=50
C10                  IF (STRREAD(is:is).EQ. ' ') then
C                     is=is+1
C                     goto 10
C                   ENDIF
C                   icharge(jtype)=icint(strread(is:80))
C                   GOTO 6025
C         ENDIF
C120   CONTINUE
C      GOTO 1015
C1005  WRITE (FU6,*) 'Error reading Gaussian 03/98/94 checkpoint file'   0701TA03
C      STOP
c
c     Now, it will look for the multiplicity, by means
c     of the title 'Multiplicity'
c
C6025  MULTID='multiplicity'
C6015  READ (FU85,1041,END=1005) STRREAD
C      DO 6120 ILINPOS=1,3
C          STRREAD=CASE(STRREAD)
C          IF (STRREAD(ILINPOS:ILINPOS+11).ne.multid) THEN
C                    GOTO 6120
C             ELSE
C                    is=50
C20                  IF (STRREAD(is:is).EQ. ' ') then
C                      is=is+1
C                      goto 20
C                    ENDIF
C                    imultipl(jtype)=icint(strread(is:80))
C                    GOTO 1025
C          ENDIF
C6120  CONTINUE
C      GOTO 6015
C1025  close (FU85)
C      return
C1041  format (a80)
C      end
c
c*****
c
c***********************************************************************
       subroutine rcodgr(icount,slinrd)
c***********************************************************************
c
c     subroutine to read in the input for the routes 
c
      implicit double precision (a-h,o-z)
      include 'param.inc'
      include 'percon.inc'
      include 'common.inc'
      include 'jaguarate.inc'
      character*80 slinrd(100)
      character * 80 string, case, tempchar
      character * 3 qend
      external case
c
      icount=0
c
c   Read the line
c
1     read (fu70,'(a80)') string
      iblnk = 1
c
c   Check if it is the 'end' line
c
      do while (iblnk.eq.1)
         i=1
         do while (iblnk.eq.1.and.i.le.80)
            if (string(i:i).ne.' ') iblnk=0
            i=i+1
         enddo
      enddo
      tempchar=string
      qend=case(tempchar(i-1:i+1))
c 
c   if it is not the end line, add the readed line to the array containing 
c   the input. Otherwise, finish reading
c
      if (qend.ne.'end') then
         slinrd(icount+1)=string
         icount=icount+1
         if (icount.eq.100) then
              write (fu6,*) 'Input too long. Maximum: 100 lines'
              stop
         endif
         goto 1
      else
         return
      endif
      return
      end
c
c*****
c
c***********************************************************************
      SUBROUTINE redinf(i,fmat,ierr)
c***********************************************************************
C
C   This subroutine reads the information about reactants, products and
C   saddle point and gives that information to polyrate, avoiding the
C   recalculation of properties that were previously calculated.
C   That information was stored in the matrices that go through the
C   "savcom" common.
C    The information on wells has also been added (JC, 11/28/97)
C
C   The subroutine checks that the geometry is the same that the one it
C   has stored; otherwise, ierr=1 and Polyrate will calculate the
C   information elsewhere.
C
      implicit double precision(a-h,o-z)
      include 'param.inc'
      include 'percon.inc'
      include 'common.inc'
      include 'jaguarate.inc'
      dimension fmat(n3tm,n3tm)
      dimension storcm(3),ancm(3)
      ierr=0
      alimg=1.0D-12
      do l=1,3
      storcm(l)=0.D0
      ancm(l)=0.D0
      enddo
      totm=0.D0
C
C     First, check the input geometry against the stored one, measuring
C     the distance between the atoms and the center of mass in both
C     cases (this is only valid for stationary points, since along the
C     reaction coordinate, global translation or rotation will cause
C     problems). If isavect=1, then the check is made for the global
C     positions instead of the positions with respect to the center
C     of masses. If isavect=2, no check is done. This is the safer option,
C     to be used when calculating anharmonicities or numerical second
C     derivatives
C
      if (isavect.eq.2) then
               ierr=1
               return
      endif
      if (isavect.eq.1) then
          if (i.ne.5) then                                              1128JC97
            do j=1,natom
              do k=1,natom
                if (j.eq.iatsv(k,i)) then
                    do l=1,3
                      if (i.lt.3) then
                         dif = x((j-1)*3+l)-geresv((j-1)*3+l)
                      else
                         dif = x((j-1)*3+l)-geprsv((j-1)*3+l)
                      endif
                      if (abs(dif).gt.alimg) then
                         ierr=1
                         return
                      endif
                    enddo
                 endif
              enddo
            enddo
          else
            do j=1,natom
                    do l=1,3
                         dif = x((j-1)*3+l)-gestsv((j-1)*3+l)
                      if (abs(dif).gt.alimg) then
                         ierr=1
                         return
                      endif
                    enddo
            enddo
          endif
       endif
      if (i.ne.5) then                                                  1128JC97
          do 1 j=1,natom
           do 2 k=1,natom
             if (j.eq.iatsv(k,i)) then
                 ams=amass(j*3)**2.0D0
                 totm=totm+ams
                do l=1,3
c
c   Calculate the center of mass of the current position (ancm) and of
c   the stored positions (storcm).
c
                 ancm(l)=ancm(l)+x((j-1)*3+l)*ams
                 if (i.lt.3) then
                  storcm(l)=storcm(l)+geresv((j-1)*3+l)*ams
                 else if (i.le.4) then                                  1128JC97
                  storcm(l)=storcm(l)+geprsv((j-1)*3+l)*ams
                 else if (i.eq.7) then                                  1128JC97
                  storcm(l)=storcm(l)+gewrsv((j-1)*3+l)*ams             1128JC97
                 else if (i.eq.8) then                                  1128JC97
                  storcm(l)=storcm(l)+gewpsv((j-1)*3+l)*ams             1128JC97
                 endif
                enddo
             endif
2          continue
1         continue
          do 3 j=1,natom
           do 4 k=1,natom
             if (j.eq.iatsv(k,i)) then
                dif=0.D0
                disnx=0.D0
                dissx=0.D0
                do l=1,3
c
c   Calculate the distance between the atoms and the center of mass
c   in both cases and compare them
c
                  disnx=disnx+(x((j-1)*3+l)-ancm(l)/totm)**2.0D0
                  if (i.lt.3) then
                   dissx=dissx+(geresv((j-1)*3+l)-storcm(l)/totm)**2.0D0
                  else if (i.le.4) then                                 1128JC97
                   dissx=dissx+(geprsv((j-1)*3+l)-storcm(l)/totm)**2.0D0
                  else if (i.eq.7) then                                 1128JC97
                   dissx=dissx+(gewrsv((j-1)*3+l)-storcm(l)/totm)**2.0D01128JC97
                  else if (i.eq.8) then                                 1128JC97
                   dissx=dissx+(gewpsv((j-1)*3+l)-storcm(l)/totm)**2.0D01128JC97
                  endif
                enddo
                  dif=sqrt(disnx)-sqrt(dissx)
                    if (abs(dif).gt.alimg) then
                        ierr=1
                        return
                    endif
             endif
4          continue
3         continue
      endif
C
      if (i.eq.5) then                                                  1128JC97
C
C   We have to do the same for the saddle point, although a bit simpler, 
C   since all the atoms are involved
C
          do 5 k=1,natom
            ams=amass(k*3)**2.0D0
            totm=totm+ams
            do l=1,3
c
c   Calculate the center of mass of the current position (ancm) and of
c   the stored positions (storcm).
c
               ancm(l)=ancm(l)+x((k-1)*3+l)*ams
               storcm(l)=storcm(l)+gestsv((k-1)*3+l)*ams
            enddo
5        continue
                dif=0.D0
                disnx=0.D0
                dissx=0.D0
          do 6 k=1,natom
            do l=1,3
c
c   Calculate the distance between the atoms and the center of mass
c   in both cases and compare them
c
                 disnx=disnx+(x((k-1)*3+l)-ancm(l)/totm)**2.0D0
                 dissx=dissx+(gestsv((k-1)*3+l)-storcm(l)/totm)**2.0D0
            enddo
                 dif=sqrt(disnx)-sqrt(dissx)
                 if (abs(dif).gt.alimg) then
                        ierr=1
                        return
                 endif
6         continue
      endif
C
C   If the program is still in this subroutine, it is because the 
C   comparison of geometries indicated that we are at the stored
C   point. Therefore, now it has to read the saved information 
C
      if (i.le.2) then
            v=enresv
            if (isup.eq.1) then
                if (i.eq.1) v=ereac1
                if (i.eq.2) v=ereac2
            endif
            do k=1,n3tm
            x(k)=geresv(k)
            dx(k)=fsresv(k)
               do l=1,n3tm
                   fmat(k,l)=hsresv(k,l)
               enddo
            enddo
      else if (i.le.4) then
            v=enprsv
            if (isup.eq.1) then
                if (i.eq.3) v=eprod1
                if (i.eq.4) v=eprod2
            endif
            do k=1,n3tm
            x(k)=geprsv(k)
            dx(k)=fsprsv(k)
               do l=1,n3tm
                 fmat(k,l)=hsprsv(k,l)
               enddo
            enddo
      else if (i.eq.5) then                                             1128JC97
            v=enstsv
            do k=1,n3tm
            x(k)=gestsv(k)
            dx(k)=fsstsv(k)
               do l=1,n3tm
                 fmat(k,l)=hsstsv(k,l)
               enddo
            enddo
      else if (i.eq.7) then                                             1128JC97
            v=enwrsv                                                    1128JC97
            do k=1,n3tm                                                 1128JC97
            x(k)=gewrsv(k)                                              1128JC97
            dx(k)=fswrsv(k)                                             1128JC97
               do l=1,n3tm                                              1128JC97
                 fmat(k,l)=hswrsv(k,l)                                  1128JC97
               enddo                                                    1128JC97
            enddo                                                       1128JC97
      else if (i.eq.8) then                                             1128JC97
            v=enwpsv                                                    1128JC97
            do k=1,n3tm                                                 1128JC97
            x(k)=gewpsv(k)                                              1128JC97
            dx(k)=fswpsv(k)                                             1128JC97
               do l=1,n3tm                                              1128JC97
                 fmat(k,l)=hswpsv(k,l)                                  1128JC97
               enddo                                                    1128JC97
            enddo                                                       1128JC97
      else                                                              1128JC97
            write (fu6,*) 'Something strange happened in REDINF. It',   1128JC97
     *                    ' entered with jtype=', i                     1128JC97
      endif
      return
      end
c
c*****
c
c***********************************************************************
      subroutine renergr(string,istrt,jtype)
c***********************************************************************
c
c     subroutine to read in the input for the route for the
c     energy calculations
c
      implicit double precision (a-h,o-z)
      include 'param.inc'
      include 'percon.inc'
      include 'common.inc'
      include 'jaguarate.inc'
      character*80 slinrd(100)
      character * 80 string, case
c     character * 3 qend                                                0701TA03
c     Variable qend was not used so it was taken out in Gaussrate 9.1.  0701TA03
      external case
c
       call rcodgr(icount,slinrd)
       if (jtype.ne.9) then                                             1128JC97
          inlene(jtype)=icount
          do i=1,icount
             enerot(jtype,i)=slinrd(i)
          enddo
       else
          do j=1,8                                                      1128JC97
          inlene(j)=icount
            do i=1,icount
               enerot(j,i)=slinrd(i)
            enddo
          enddo
       endif
c
      return
      end
c
c*****
c
c***********************************************************************
       subroutine rfirstgr(string,istrt,jtype)
c***********************************************************************
c
c     subroutine to read in the input for the route for the
c     first derivative calculations
c
      implicit double precision (a-h,o-z)
      include 'param.inc'
      include 'percon.inc'
      include 'common.inc'
      include 'jaguarate.inc'
      character*80 slinrd(100)
      character * 80 string, case
      external case
c
       call rcodgr(icount,slinrd)
       if (jtype.ne.9) then                                             1128JC97
           inlfrs(jtype)=icount
           do i=1,icount
                  firrot(jtype,i)=slinrd(i)
           enddo
       else
          do j=1,8                                                      1128JC97
          inlfrs(j)=icount
            do i=1,icount
               firrot(j,i)=slinrd(i)
            enddo
          enddo
       endif
c
      return
      end
c
c*****
c
c***********************************************************************
c     subroutine rgenbas(string,istrt,jtype)
c***********************************************************************
c
c     subroutine to read in a general basis set
c
c     implicit double precision (a-h,o-z)
c     include 'param.inc'
c     include 'percon.inc'
c     include 'common.inc'
c     include 'gaussrate.inc'
c     character*80 slinrd(100)
c     character * 80 string, case
c     character * 3 qend                                                0701TA03
c     Variable qend was not used so it was taken out in Gaussrate 9.1.  0701TA03
c     external case
c
c      call rcodgr(icount,slinrd)
c         inlbas=icount
c           do i=1,icount
c              basblk(i)=slinrd(i)
c           enddo
c
c     return
c     end
c
c*****
c
c***********************************************************************
       subroutine rgengr(string,iend,istrt)
c***********************************************************************
c
c     subroutine to read in the input in the general section. By
c     now, it only includes the restart option
c
      implicit double precision (a-h,o-z)
      include 'param.inc'
      include 'percon.inc'
      include 'common.inc'
      include 'jaguarate.inc'
      character * 80 string
c
c     set the defaults:
c
c     GRRESTART: ON
      irest=1
c
c     RSTTOL: 1.0D-8                                                     0825JC00
      fallout=1.0D-8                                                     0825JC00
c
c     read in first keyword in this section
c
      call rlingr(string,istrt,isect,iend)
c
      do while (isect.eq.0.and.iend.eq.0)
c GRRESTART
         if (string(istrt:istrt+4).eq.'jrres') then
            irest = 1
c NOGRRESTART
         else if (string(istrt:istrt+6).eq.'nojrres') then
            irest = 0
c RSTTOL                                                                 0825JC00
         else if (string(istrt:istrt+5).eq.'rsttol') then                0825JC00
            call rword(string,istrt,ierr)                                0825JC00
            if(ierr.ne.1) then                                           0825JC00
               fallout = cfloat(string(istrt:80))                        0825JC00
               write(fu6,*)'WARNING: RSTTOL changed to', fallout         0825JC00
            else                                                         0825JC00
               write(fu6,*)'ERROR reading keyword RSTTOL in esp.fu70'    0825JC00
            stop                                                         0825JC00
            end if                                                       0825JC00
         else
            write(fu6,1200) string(istrt:80)
            stop
         end if
         call rlingr(string,istrt,isect,iend)
      end do
c
1200  format(3x,'Error:  the following is not a valid keyword in the',
     *       ' jrgeneral section',/A80)
c
      return
      end
c
c*****
c
c***********************************************************************
      SUBROUTINE rinpgr
c***********************************************************************
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      include 'param.inc'
      include 'percon.inc'
      include 'common.inc'
      include 'jaguarate.inc'
c     character * 80 string,case                                        0701TA03
c     Variable case was not used so it was taken out in Gaussrate 9.1.  0701TA03
      character * 80 string                                             0701TA03
      inlcer=0
      inlbas=0
      do i=1,8                                                          1128JC97
       inlscr(i)=1
       inlfrs(i)=1
       inlene(i)=1
       secrot(i,1)='&gen isymm=0 iunit=2 basis=STO-3G ifreq=1 igeopt=
     *-1 &'
       firrot(i,1)='&gen isymm=0 iunit=2 basis=STO-3G igeopt=-1 &'
       enerot(i,1)='&gen isymm=0 iunit=2 basis=STO-3G &'
      enddo
c
C     This subroutine reads the input of the Jaguarate options. It
C     starts reading if it is a Jrate restart run. If it is, open
c     the reading file and the writing file.
c
      call rlingr(string,istrt,isect,iend)
c
C     Those error check lines are equivalents to those when reading fu5
c
      if (iend.eq.1) then
         write(fu6,1000)
1000     format(1x,'error:  input file was empty')
         stop
      end if
c
      if (isect.ne.1) then
         write(fu6,1100)
1100     format(1x,'error:  first non-comment, non-blank line must be'
     *             ,' a section header that begins with a *')
         stop
      end if
c
c     Now, it reads the sections in grate input
c
      do while (iend.eq.0)
         j = istrt + 1
         if (string(j:j+5).eq.'jrgene') then
               call rgengr(string,iend,istrt)
         else if (string(j:j+7).eq.'jrreact1') then
               call rstatgr(string,iend,istrt,1)
         else if (string(j:j+7).eq.'jrreact2') then
               call rstatgr(string,iend,istrt,2)
         else if (string(j:j+6).eq.'jrprod1') then
               call rstatgr(string,iend,istrt,3)
         else if (string(j:j+6).eq.'jrprod2') then
               call rstatgr(string,iend,istrt,4)
         else if (string(j:j+6).eq.'jrstart') then
               call rstatgr(string,iend,istrt,5)
         else if (string(j:j+5).eq.'jrpath') then
               call rstatgr(string,iend,istrt,6)
         else if (string(j:j+6).eq.'jrwellr') then                      1128JC97
               call rstatgr(string,iend,istrt,7)                        1128JC97
         else if (string(j:j+6).eq.'jrwellp') then                      1128JC97
               call rstatgr(string,iend,istrt,8)                        1128JC97
         else if (string(j:j+7).eq.'jrcommon') then
               call rstatgr(string,iend,istrt,9)                        1128JC97
         else
               write(fu6,1200) string(j-1:80)
               stop
         end if
      end do
c
1200  format(3x,'Error:  the following is not a valid section name',
     *         /A80)
c
      if (irest.eq.1) then
         OPEN (UNIT=FU83,FILE='esp.fu83')
         OPEN (UNIT=FU84,STATUS='NEW',FILE='esp.fu84')
         write(fu6,2001)
      endif
c
2001     format(/,5x,'Jaguarate restart calculation',/)
C
      RETURN
      END
C
C*****
C
C***********************************************************************
C     subroutine rlincer(string,istrt,jtype)
C***********************************************************************
C
C     subroutine to read in the link0 options
C
C     implicit double precision (a-h,o-z)
C     include 'param.inc'
C     include 'percon.inc'
C     include 'common.inc'
C     include 'gaussrate.inc'
C     character*80 slinrd(100)
C     character * 80 string, case
C     character * 3 qend                                                0701TA03
C     Variable qend was not used so it was taken out in Gaussrate 9.1.  0701TA03
C     external case
C
C      call rcodgr(icount,slinrd)
C         inlcer=icount
C           do i=1,icount
C              cerblk(i)=slinrd(i)
C           enddo
C
C     return
C     end
c
c*****
c
c***********************************************************************
      SUBROUTINE rlingr(string,istrt,isect,iend)
c***********************************************************************
C
c     This subroutine is equivalent to rline in interface.f, but a bit
c     modified for reading from unit fu70
c
c      istrt:   location in 80 character string that is the first non-blank
c      isect:   a 1 is returned if the first non-blank character is a *
c               signalling the start of a section
c      iend:    a 1 is returned if the end of file 5 is encountered
c      string:  80 character string that is the contents of the current line
c
      implicit double precision (a-h,o-z)
c
      include 'param.inc'
      include 'percon.inc'
      include 'common.inc'
      include 'jaguarate.inc'
c
      character * 80 string,case
      external case
c
1000  format(A80)
c
c       initialize variables
c
      isect = 0
      iend = 0
      iblnk = 1
c
c     read in next line in file 5 - find first non-blank character and
c     determine if it is a comment (# sign signals a comment).  Once a
c     comment is found the rest of the line is skipped (by setting the
c     counter to the last character [i=80])
c
      do while (iblnk.eq.1)
         read(fu70,1000,end=9999) string
         i = 1
         do while (iblnk.eq.1.and.i.le.80)
            if (string(i:i).ne.' ') then
               if (string(i:i).ne.'#') then
                   iblnk = 0
                else
                   i = 80
                end if
            end if
            i = i+1
         end do
       end do
c
c      set location of first non-blank character
       istrt = i-1
c
c      check if it is a section header (first character a *)
c
       if (string(istrt:istrt).eq.'*') isect=1
c
c      strip of any trailing comments on the line
c
       do 1 j=istrt,80
1        if (string(j:j).eq.'#') string(j:80) = ' '
c
c      change to lower case
c
       string = case(string)
       return
9999   iend=1
       return
       end
c
c*****
c
c***********************************************************************
       subroutine rsecgr(string,istrt,jtype)
c***********************************************************************
c
c     subroutine to read in the input for the route for the
c     second derivative calculations
c
      implicit double precision (a-h,o-z)
      include 'param.inc'
      include 'percon.inc'
      include 'common.inc'
      include 'jaguarate.inc'
      character*80 slinrd(100)
      character * 80 string, case
c     character * 3 qend                                                0701TA03
c     Variable qend was not used so it was taken out in Gaussrate 9.1.  0701TA03
      external case
c
       call rcodgr(icount,slinrd)
       if (jtype.ne.9) then                                             1128JC97
          inlscr(jtype)=icount
          do i=1,icount
             secrot(jtype,i)=slinrd(i)
          enddo
       else
          do j=1,8                                                      1128JC97
          inlscr(j)=icount
            do i=1,icount
               secrot(j,i)=slinrd(i)
            enddo
          enddo
       endif
c
      return
      end
c
c*****
c
c***********************************************************************
       subroutine rstatgr(string,iend,istrt,jtype)
c***********************************************************************
c
c     subroutine to read in the input for all the stationary points
c
      implicit double precision (a-h,o-z)
      include 'param.inc'
      include 'percon.inc'
      include 'common.inc'
      include 'jaguarate.inc'
      character * 80 string
c
c     Jtype = 1  first reactant
c     Jtype = 2  second reactant
c     Jtype = 3  first product
c     Jtype = 4  second product
c     Jtype = 5  saddle point or starting stationary point
c     Jtype = 6  all the points along the path
c     Jtype = 7  reactant well
c     Jtype = 8  product well
c     Jtype = 9  all the stationary points and points along the path
c
c     read in first keyword in this section
c
      call rlingr(string,istrt,isect,iend)
c
      do while (isect.eq.0.and.iend.eq.0)
c
C GROPT
         if (string(istrt:istrt+4).eq.'jropt') then
            write (fu6,1150)                                            1128JC97
            stop                                                        1128JC97
C NOGROPT
         else if (string(istrt:istrt+6).eq.'nojropt') then
            write (fu6,1150)                                            1128JC97
            stop                                                        1128JC97
C CHARGE
C        else if (string(istrt:istrt+5).eq.'charge') then
C           call rword(string,istrt,ierr)
C           if(ierr.ne.1) then
C             if (jtype.ne.9.and.jtype.ne.6) then                       1128JC97
C              icharge(jtype) = icint(string(istrt:80))
C             else 
C              write(fu6,1050)
C              stop
C             endif
C           else
C              write(fu6,1100)
C              stop
C           end if
C MULTIPLICITY
C        else if (string(istrt:istrt+5).eq.'multip') then
C           call rword(string,istrt,ierr)
C           if(ierr.ne.1) then
C             if (jtype.ne.9.and.jtype.ne.6) then                       1128JC97
C                imultipl(jtype) = icint(string(istrt:80))
C             else 
C                write(fu6,1050)
C                stop
C             endif
C           else
C              write(fu6,1100)
C              stop
C           end if
C GRLINK0 (only allowed in GRCOMMON)
C        else if (string(istrt:istrt+6).eq.'grlink0') then
C           if (jtype.eq.9) then                                        1128JC97
C             call rlincer(string,istrt,jtype)
C           else
C             write(fu6,1250)
C             stop
C           endif
C GRBASIS (only allowed in GRCOMMON)
C        else if (string(istrt:istrt+6).eq.'grbasis') then
C           if (jtype.eq.9) then                                        1128JC97
C             call rgenbas(string,istrt,jtype)
C           else
C             write(fu6,1250)
C             stop
C           endif
C GRENER
         else if (string(istrt:istrt+5).eq.'jrener') then
            call renergr(string,istrt,jtype)
C GRFIRST
         else if (string(istrt:istrt+6).eq.'jrfirst') then
            call rfirstgr(string,istrt,jtype)
C GRSEC
         else if (string(istrt:istrt+4).eq.'jrsec') then
            call rsecgr(string,istrt,jtype)
         else
            write(fu6,1200) string(istrt:80)
            stop
         end if
         call rlingr(string,istrt,isect,iend)
      end do
c
1050  format(3x,'Error:  MULTIPLICITY and CHARGE can not be in',
     *       ' COMMON or PATH sections',/)
1100  format(3x,'Error reading CHARGE or MULTIPLICITY',/)
1150  format(3x,'Error:  GROPT and NOGROPT are not longer valid',
     *       ' keywords',/)
1200  format(3x,'Error:  the following is not a valid keyword in',
     *       ' this section',/A80)
1250  format(3x,'Error:  grlink0 and grbasis are only allowerd in',
     *       ' the common section',/)
c
      return
      end
c
c*****
c
c***********************************************************************
      SUBROUTINE setup
c***********************************************************************
C
C   This subroutine writes a title for the polyrate long output and
C   call the subroutine for reading Jaguarate input
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      include 'param.inc'
      include 'percon.inc'
      include 'common.inc'
      include 'jaguarate.inc'
C
      open (unit=fu70,status='old',file='esp.fu70')
      call rinpgr
      close (fu70)
C
         WRITE (FU6,100) 
C
100   FORMAT (/,2X,T5,'Jaguar will carry out the electronic structure'
     *        /,2X,T5,'calculations needed for obtaining the required'
     *        /,2X,T5,'data for the POLYRATE calculation',/)
C
C     Check compatibility of options and set gropt
C
      if (isup.eq.0) then
         write (fu6,200)
         stop 'setup 1'
      endif
c
c     OHOOK optimization:
c
      if (igpot.eq.1) then                                              1203JC97
         do 10 i=1,8                                                    1203JC97
           if (i.eq.6) goto 10                                          1203JC97
           if (irepr(i).eq.0) goto 10                                   1203JC97
c
c     check all the systems to be optimized have initgeo=hooks
c     and set ioptgr=1
c
           if (istatu(i).eq.0) then                                     1203JC97
              if (initg(i).eq.1) then                                   1203JC97
                  ioptgr(i) = 1                                         1203JC97
              else                                                      1203JC97
                  write (fu6,300) i                                     1203JC97
                  stop 'setup 2'                                        1203JC97
              endif                                                     1203JC97
            else                                                        1203JC97
c
c     check all the systems not to be optimized have initgeo=geom 
c     and set ioptgr=0
c
              if (initg(i).eq.0) then                                   1203JC97
                  ioptgr(i) = 0                                         1203JC97
              else                                                      1203JC97
                  write (fu6,400) i                                     1203JC97
                  stop 'setup 3'                                        1203JC97
              endif                                                     1203JC97
            endif                                                       1203JC97
10       continue                                                       1203JC97
c
c     BFGS or NH optimizations
c
      else                                                              1203JC97
         do 20 i=1,8                                                    1203JC97
           if (i.eq.6) goto 20                                          1203JC97
           if (irepr(i).eq.0) goto 20                                   1203JC97
c
c     check all the systems have initgeo=geom 
c
           if (initg(i).eq.0) then                                      1203JC97
                  ioptgr(i) = 0                                         1203JC97
           else                                                         1203JC97
                  write (fu6,500) i                                     1203JC97
                  stop 'setup 3'                                        1203JC97
           endif                                                        1203JC97
20       continue                                                       1203JC97
      endif                                                             1203JC97
C
c
200   FORMAT (/,2X,T5,'Error: Jaguarate requires the no-supermolecule'
     *        /,2X,T5,'mode. The program will stop',/)
300   FORMAT (/,2x,T5,'Error: OHOOK has been selected for the '         1203JC97
     *        /,2X,T5,'optimization, and the species ', I3,' is going'  1203JC97
     *        /,2X,T5,'to be optimized, so the geometry should be'      1203JC97
     *        /,2X,T5,'given in HOOKS (INITGEO=HOOKS).'/)               1203JC97   
400   FORMAT (/,2x,T5,'Error: OHOOK has been selected for the '         1203JC97   
     *        /,2X,T5,'optimization, and the species ', I3,' is not '   1203JC97   
     *        /,2X,T5,'going to be optimized, so the geometry should '  1203JC97     
     *        /,2X,T5,'be given in FU5 (INITGEO=GEOM).'/)               1203JC97 
500   FORMAT (/,2x,T5,'Error: BFGS or NR has been selected for the '    1203JC97   
     *        /,2X,T5,'optimization, so for the species ', I3,' the'    1203JC97   
     *       /,2X,T5,'geometry should be given in FU5 (INITGEO=GEOM).'/)1203JC97 
C
      RETURN
      END
c
c*****
c
c***********************************************************************
      SUBROUTINE store(V,COORD,DX,FMAT,INDOP)
c***********************************************************************
C
C    This subroutine checks if the first derivative calculation is going 
C    to be carried out on a point where the second derivative has been
C    calculated right before this calcularion. If it is, it reads the
C    first derivative instead of calculating it, so that computing time
C    is saved.
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      include 'param.inc'
      include 'percon.inc'
      include 'jaguarate.inc'
      COMMON /STRCOM/ STOREV,STORECOORD(N3TM),STOREDX(N3TM),
     *STOREFMAT(N3TM,N3TM)
      DIMENSION COORD(N3TM),DX(N3TM),FMAT(N3TM,N3TM)
      difall=1.0D-10
      if (indop.eq.1) then
         storev=v
         do instr=1,n3tm
            storecoord(instr)=coord(instr)
            storedx(instr)=dx(instr)
              do jnstr=1,n3tm
               storefmat(instr,jnstr)=fmat(instr,jnstr)
              enddo
         enddo
      else
        do incont=1,n3tm
             dif=COORD(incont)-STORECOORD(incont)
             if (abs(dif).gt.difall) goto 101
        enddo
        v=storev
        do incont=1,n3tm
           dx(incont)=storedx(incont)
        enddo
         indop=1
      endif
101   return
      end
